package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class TestBug {

    private static final String PU_NAME = "test-jpa-pu-derby";

    private static final long ID = 1L;
    private static final String NAME = "Master 1";

    @Test
    public void persistTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            em.getTransaction().begin();
            boolean value = false;
            TestEntityMaster entity = new TestEntityMaster(10, "Persist Master 1");
            em.persist(entity);
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            List<TestEntityMaster> testEntities = em.createQuery("SELECT e FROM TestEntityMaster AS e").getResultList();
            assertTrue(testEntities.size() > 0);
            assertEquals(ID, testEntities.get(0).getId());
            assertEquals(NAME, testEntities.get(0).getName());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void findTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            TestEntityMaster testMaster = em.find(TestEntityMaster.class, ID);
            assertNotNull(testMaster);
            assertEquals(ID, testMaster.getId());
            assertEquals(NAME, testMaster.getName());

            List<TestEntityDetail> testDetails = testMaster.getDetails();
            assertNotNull(testDetails);
            assertEquals(2, testDetails.size());

        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }
}
