package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import org.junit.jupiter.api.Test;

import java.time.LocalDateTime;
import java.util.Properties;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

public class TestDDL {

    private static final String PU_NAME = "test-jpa-pu-ddl";

    private static final UUID ID_UUID = UUID.fromString("4ae8976b-635f-46dc-937a-6016db49bd96");
    private final LocalDateTime LOCAL_DATE_TIME = LocalDateTime.of(2024, 10, 17, 9, 30, 29);


    @Test
    public void persistTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            em.getTransaction().begin();
            DDLEntity ddlEntity = new DDLEntity(ID_UUID, LOCAL_DATE_TIME);
            em.persist(ddlEntity);
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void findTest() {
        Properties properties = new Properties();
        properties.setProperty("jakarta.persistence.schema-generation.database.action", "none");
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME, properties);
        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            DDLEntity entity = em.find(DDLEntity.class, ID_UUID);
            assertNotNull(entity);
            assertEquals(ID_UUID, entity.getId());
            assertEquals(LOCAL_DATE_TIME, entity.getSimpleLocalDateTime());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTest() {
        Properties properties = new Properties();
        properties.setProperty("jakarta.persistence.schema-generation.database.action", "none");
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME, properties);
        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            DDLEntity entity = em.createQuery("SELECT OBJECT(o) FROM DDLEntity o WHERE o.id = :idParam", DDLEntity.class).
            setParameter("idParam", ID_UUID).
            getSingleResult();
            assertNotNull(entity);
            assertEquals(ID_UUID, entity.getId());
            assertEquals(LOCAL_DATE_TIME, entity.getSimpleLocalDateTime());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void extractTest() {
        Properties properties = new Properties();
        properties.setProperty("jakarta.persistence.schema-generation.database.action", "none");
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME, properties);
        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Integer result = em.createQuery("SELECT EXTRACT(YEAR FROM o.simpleLocalDateTime) FROM DDLEntity o WHERE o.id = :idParam", Integer.class).
                    setParameter("idParam", ID_UUID).
                    getSingleResult();
            assertNotNull(result);
            assertEquals(LOCAL_DATE_TIME.getYear(), result);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

}
