package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.EntityTransaction;
import jakarta.persistence.Persistence;
import org.junit.jupiter.api.Test;

import java.math.BigInteger;
import java.time.Duration;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.ArrayList;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;

public class TestDemographicInfoBug2191 {

    private static final String PU_NAME = "test-jpa-pu-ddl";


    @Test
    // @SkipIfSysProp(DB_DB2) // Reference issue: https://github.com/OpenLiberty/open-liberty/issues/29443
    public void testBug() throws Exception {

        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        EntityManager em = entityManagerFactory.createEntityManager();
        EntityTransaction tx = em.getTransaction();


        deleteAllEntities(DemographicInfoEntity.class);

        ZoneId ET = ZoneId.of("America/New_York");
        Instant when = ZonedDateTime.of(2022, 4, 29, 12, 0, 0, 0, ET)
                .toInstant();

        DemographicInfoEntity US2022 = DemographicInfoEntity.of(2022, 4, 29, 132250000, 6526909395140.41, 23847245116757.60);
        DemographicInfoEntity US2007 = DemographicInfoEntity.of(2007, 4, 30, 121090000, 3833110332444.19, 5007058051986.64);

        List<BigInteger> results;

        tx.begin();
        em.persist(US2022);
        em.persist(US2007);
        tx.commit();

        Thread.sleep(Duration.ofSeconds(1).toMillis());

        tx.begin();
        results = em
                .createQuery("SELECT o FROM DemographicInfoEntity o WHERE (o.publicDebt BETWEEN ?1 AND ?2) ORDER BY o.publicDebt",
                        BigInteger.class)
                .setParameter(1, 0)
                .setParameter(2, 600000000000000.00)
                .getResultList();
        tx.commit();

        assertNotNull(results, "Query should not have returned null");
        assertFalse(results.isEmpty(), "Query should not have returned an empty list"); // Recreate
        assertEquals(2, results.size(),"Query should not have returned more than one result");
    }

    private void deleteAllEntities(Class<DemographicInfoEntity> demographicInfoClass) {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        EntityManager em = entityManagerFactory.createEntityManager();
        try {
            em.getTransaction().begin();
            em.createNativeQuery("DELETE FROM TEST_TAB_DEMOGRAPHIC_INFO").executeUpdate();
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }


    }
}
