package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

public class TestEmbedded {

    private static final String PU_NAME = "test-jpa-embedded-pu-derby";

    private final int ID_INT = 1;
    private final String ID_STRING = "a";
    private final String NAME1 = "abcde";
    private final String NAME2 = "xyz";

    @Test
    public void persistTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            em.getTransaction().begin();

            em.createNativeQuery("DELETE FROM TEST_TAB_EMBEDDED").executeUpdate();

            EmbeddedEntity entity = new EmbeddedEntity();
            entity.setId(new EmbeddedPK(ID_INT, ID_STRING));
            entity.setEmbeddedAttribute(new EmbeddedAttribute(NAME1, NAME2));
            em.persist(entity);
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void findTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            EmbeddedEntity entity = em.find(EmbeddedEntity.class, new EmbeddedPK(ID_INT, ID_STRING));
            assertNotNull(entity);

            assertEquals(ID_INT, entity.getId().getId_int());
            assertEquals(ID_STRING, entity.getId().getId_string());
            assertEquals(NAME1, entity.getEmbeddedAttribute().getName1());
            assertEquals(NAME2, entity.getEmbeddedAttribute().getName2());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            List<EmbeddedEntity> result = em.createQuery("SELECT OBJECT(o) FROM EmbeddedEntity o WHERE o.embeddedAttribute = :embeddedAttribute").
            setParameter("embeddedAttribute", new EmbeddedAttribute(NAME1, NAME2)).
            getResultList();

            for (EmbeddedEntity entity: result) {
                assertNotNull(entity);

                assertEquals(ID_INT, entity.getId().getId_int());
                assertEquals(ID_STRING, entity.getId().getId_string());
                assertEquals(NAME1, entity.getEmbeddedAttribute().getName1());
                assertEquals(NAME2, entity.getEmbeddedAttribute().getName2());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }
}
