package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import jakarta.persistence.PersistenceUnitUtil;
import jakarta.persistence.Query;
import org.checkerframework.checker.units.qual.N;
import org.junit.jupiter.api.Test;

import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;

public class TestIdVersionJPQL {

    private static final String PU_NAME = "test-jpa-pu-derby";

    private final long ID = 1;
    private final IdVersionPK ID_COMPOSITE = new IdVersionPK(1, 11);

    private final String NAME = "abcde";
    private final int VERSION = 11;


    @Test
    public void queryOBJECTTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT OBJECT(o) FROM IdVersionEntity o");
            List<IdVersionEntity> result = query.getResultList();

            for (IdVersionEntity entity : result) {
                assertNotNull(entity);

                assertEquals(ID, entity.getId());
                assertEquals(NAME, entity.getName());
                assertEquals(VERSION, entity.getVersion());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID1Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT ID(o) FROM IdVersionEntity o");
            List<Long> result = query.getResultList();

            for (Long value : result) {
                assertEquals(ID, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID2Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT o.id, o.version, ID(o) FROM IdVersionEntity o");
            List<Object[]> result = query.getResultList();

            for (Object[] value : result) {
                assertEquals(ID, value[0]);
                assertEquals(VERSION, value[1]);
                assertEquals(ID, value[2]);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID3Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT o.id FROM IdVersionEntity o WHERE ID(o) = " + ID);
            List<Long> result = query.getResultList();

            for (Long value : result) {
                assertEquals(ID, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID4Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT ID(o) FROM IdVersionEntity o WHERE ID(o) = " + ID);
            List<Long> result = query.getResultList();

            for (Long value : result) {
                assertEquals(ID, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID5Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT o.id, o.version, ID(o) FROM IdVersionEntity o WHERE ID(o) = " + ID);
            List<Object[]> result = query.getResultList();

            for (Object[] value : result) {
                assertEquals(ID, value[0]);
                assertEquals(VERSION, value[1]);
                assertEquals(ID, value[2]);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID6Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT NEW com.oracle.jpa.bugtest.IdVersionNewEntity(id, name, version) FROM IdVersionEntity WHERE id = :idParam ORDER BY name DESC, id ASC");
            query.setParameter("idParam", ID);
            IdVersionNewEntity result = (IdVersionNewEntity) query.getSingleResult();
/*
            assertEquals(ID, result.getId());
            assertEquals(VERSION, result.getVersion());
            assertEquals(NAME, result.getName());
*/
            assertEquals(ID, result.id);
            assertEquals(VERSION, result.version);
            assertEquals(NAME, result.name);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID7Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT EXP(CAST(ID(this) AS INTEGER)) FROM IdVersionEntity WHERE ID(this) = " + ID, Double.class);
//            Query query = em.createQuery("SELECT EXP(ID(this)) FROM IdVersionEntity WHERE ID(this) = " + ID, Double.class);
            Double result = (Double) query.getSingleResult();

            assertEquals(2.7182818284590455, result);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

//    @Test
    public void queryID6DeleteTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            em.getTransaction().begin();
            Query query = em.createQuery("DELETE FROM IdVersionEntity o WHERE ID(o) = " + ID);
            int result = query.executeUpdate();
            em.getTransaction().commit();
            assertEquals(1, result);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryID7CompositePKTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT o.name, o.id, ID(o) FROM IdVersionCompositePKEntity o");
            List<Object[]> result = query.getResultList();

            for (Object[] value : result) {
                assertEquals("xyz", value[0]);
                assertEquals(ID_COMPOSITE, value[1]);
                assertEquals(ID_COMPOSITE, value[2]);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryVERSION1Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT VERSION(o) FROM IdVersionEntity o");
            List<Integer> result = query.getResultList();

            for (Integer value : result) {
                assertEquals(VERSION, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryVERSION2Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT o.id, o.version, VERSION(o) FROM IdVersionEntity o");
            List<Object[]> result = query.getResultList();

            for (Object[] value : result) {
                assertEquals(ID, value[0]);
                assertEquals(VERSION, value[1]);
                assertEquals(VERSION, value[2]);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryVERSION3Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT o.id FROM IdVersionEntity o WHERE VERSION(o) = " + VERSION);
            List<Long> result = query.getResultList();

            for (Long value : result) {
                assertEquals(ID, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryVERSION4Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT VERSION(o) FROM IdVersionEntity o WHERE VERSION(o) = " + VERSION);
            List<Integer> result = query.getResultList();

            for (Integer value : result) {
                assertEquals(VERSION, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryVERSION5Test() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
            Query query = em.createQuery("SELECT o.id, o.version, VERSION(o) FROM IdVersionEntity o WHERE VERSION(o) = " + VERSION);
            List<Object[]> result = query.getResultList();

            for (Object[] value : result) {
                assertEquals(ID, value[0]);
                assertEquals(VERSION, value[1]);
                assertEquals(VERSION, value[2]);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryVERSION6WithThisTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        PersistenceUnitUtil puu = entityManagerFactory.getPersistenceUnitUtil();

        try {
/*
            Query query = em.createQuery("SELECT VERSION(this) FROM IdVersionEntity WHERE name = :nameParam");
            query.setParameter("nameParam", NAME);
*/
            Query query = em.createQuery("SELECT VERSION(this) FROM IdVersionEntity WHERE name = '" + NAME + "'");
            List<Integer> result = query.getResultList();

            for (Integer value : result) {
                assertEquals(VERSION, value);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

}
