package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.EntityTransaction;
import jakarta.persistence.Persistence;
import org.junit.jupiter.api.Test;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class TestNEWWithCollectionInConstructor {

    private static final String PU_NAME = "test-jpa-pu-rating-derby";

    @Test
    public void init() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        EntityManager em = entityManagerFactory.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        List<Rating> ratings = null;

        try {
            tx.begin();
            em.createQuery("DELETE FROM Rating").executeUpdate();
            tx.commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
        }

        try {
            Set<String> comments1 = new HashSet<>();
            comments1.add("comment1.1");
            comments1.add("comment1.2");
            Rating r1 = new Rating(1, Rating.Item.of("Item1", 11.11f,11f), 1, Rating.Reviewer.of("FirstName1", "LastName1", "firstname1.lastname1@email.com"), comments1);

            tx.begin();
            em.persist(r1);
            tx.commit();

        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    @Test
    // @Ignore("Reference issue: https://github.com/eclipse-ee4j/eclipselink/issues/2264")
    public void testSelect() {

        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        EntityManager em = entityManagerFactory.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        List<String> comments = null;

        tx.begin();
        try {

            comments = em.createQuery("SELECT o.comments FROM Rating o WHERE o.id = :idParam",
                            String.class)
                    .setParameter("idParam", 1)
                    .getResultList();

            assertEquals(2, comments.size());

            tx.commit();
        } catch (Exception e) {
            tx.rollback();
            throw e;
        } finally {
            em.close();
            entityManagerFactory.close();
        }
    }

    @Test
    // @Ignore("Reference issue: https://github.com/eclipse-ee4j/eclipselink/issues/2264")
    public void testSelectNew() {

        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        EntityManager em = entityManagerFactory.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        Rating rating = null;

        tx.begin();
        try {

            rating = em.createQuery("SELECT NEW com.oracle.jpa.bugtest.Rating(o.id, o.item, o.numStars, o.reviewer, o.comments) FROM Rating o WHERE o.id = :idParam",
                            Rating.class)
                    .setParameter("idParam", 1)
                    .getSingleResult();

            assertEquals(1, rating.id);

            tx.commit();
        } catch (Exception e) {
            tx.rollback();
            throw e;
        } finally {
            em.close();
            entityManagerFactory.close();
        }
    }

}
