package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import jakarta.persistence.Query;
import org.eclipse.persistence.jpa.jpql.WordParser;
import org.eclipse.persistence.jpa.jpql.parser.Expression;
import org.junit.jupiter.api.Test;


import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertTrue;

public class TestNoAliasJPQL {

    private static final String PU_NAME = "test-jpa-pu-derby";

    private final long ID = 1;
    private final String NAME = "abcde";
//    private final Instant COLLECTED_ON = Instant.parse("2024-05-27T10:15:30.00Z");
    ZoneId ET = ZoneId.of("UTC");
    private final Instant COLLECTED_ON = ZonedDateTime.of(2024, 5, 27, 10, 15, 30, 0, ET)
            .toInstant();

    //    @Test
    public void persistTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            em.getTransaction().begin();

            em.createNativeQuery("DELETE FROM TEST_TAB_NOALIAS").executeUpdate();

            NoAliasEntity entity = new NoAliasEntity();
            entity.setId(ID);
            entity.setName(NAME);
            entity.setCollectedOn(COLLECTED_ON);
            em.persist(entity);
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void findTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            NoAliasEntity entity = em.find(NoAliasEntity.class, ID);
            assertNotNull(entity);

            assertEquals(ID, entity.getId());
            assertEquals(NAME, entity.getName());
            assertEquals(COLLECTED_ON, entity.getCollectedOn());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT o FROM NoAliasEntity o");
            List<NoAliasEntity> result = query.getResultList();

            for (NoAliasEntity entity : result) {
                assertNotNull(entity);

                assertEquals(ID, entity.getId());
                assertEquals(NAME, entity.getName());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryNoAliasTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT o FROM NoAliasEntity");
            List<NoAliasEntity> result = query.getResultList();

            for (NoAliasEntity entity : result) {
                assertNotNull(entity);

                assertEquals(ID, entity.getId());
                assertEquals(NAME, entity.getName());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryNoAliasByInstantTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT this.id FROM NoAliasEntity WHERE this.collectedOn = :collectedOn_param");
            query.setParameter("collectedOn_param", COLLECTED_ON);
            List<Long> results = query.getResultList();

            for (Long result : results) {
                assertNotNull(result);

                assertEquals(ID, result);
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestOBJECT() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT OBJECT(o) FROM NoAliasEntity");
            List<NoAliasEntity> result = query.getResultList();

            for (NoAliasEntity entity : result) {
                assertNotNull(entity);

                assertEquals(ID, entity.getId());
                assertEquals(NAME, entity.getName());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestCOUNT() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT COUNT(o) FROM NoAliasEntity", Long.class);
            Long result = (Long)query.getSingleResult();

            assertEquals(1L, result);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestCASTCOUNT() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT CAST(COUNT(o) AS SMALLINT) FROM NoAliasEntity", Long.class);
            Integer result = (Integer) query.getSingleResult();

            assertEquals(1, result);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestCASTCASTCOUNT() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT CAST(CAST(COUNT(o) AS SMALLINT) AS BIGINT) FROM NoAliasEntity", Long.class);
            Long result = (Long) query.getSingleResult();

            assertEquals(1, result);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestThisWhere01() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
//OK            Query query = em.createQuery("SELECT this FROM NoAliasEntity WHERE id = :idParam AND UPPER(name) NOT LIKE UPPER(:nameParam)", Long.class);
//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE id = :idParam AND UPPER(name) NOT LIKE UPPER(:nameParam)", Long.class);
//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE id = :idParam AND UPPER(name) NOT LIKE UPPER(:nameParam) ORDER BY id DESC, name", Long.class);
//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE id = :idParam AND UPPER(name) NOT LIKE UPPER(:nameParam) AND UPPER(name) NOT LIKE 'dgdgs' ORDER BY id DESC, name", Long.class);
            Query query = em.createQuery("FROM NoAliasEntity WHERE (id <= :idParam AND UPPER(name) NOT LIKE UPPER(:nameParam) AND UPPER(name) NOT LIKE 'dgdgs') ORDER BY id DESC, name", Long.class);
            query.setParameter("idParam", ID);
            query.setParameter("nameParam", "asdadads");
            NoAliasEntity result = (NoAliasEntity)query.getSingleResult();

            assertEquals(ID, result.getId());
            assertEquals(NAME, result.getName());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    @Test
    public void queryTestThisWhere02() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE :rate * id <= :max ORDER BY id DESC, name", Long.class);
            Query query = em.createQuery("FROM NoAliasEntity WHERE (:rate * id <= :max AND :rate * id >= :min) ORDER BY name");
            query.setParameter("max", 1);
            query.setParameter("min", 1);
            query.setParameter("rate", 1);
            NoAliasEntity result = (NoAliasEntity)query.getSingleResult();

            assertEquals(ID, result.getId());
            assertEquals(NAME, result.getName());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestSimpleWhere() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            Query query = em.createQuery("SELECT e FROM NoAliasEntity e WHERE e.id = " + ID);
            NoAliasEntity result = (NoAliasEntity)query.getSingleResult();

            assertEquals(ID, result.getId());
            assertEquals(NAME, result.getName());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestSimpleWhereWithBraces() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
//OK            Query query = em.createQuery("SELECT e FROM NoAliasEntity e WHERE (e.id = " + ID + ")");
//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE (this.id = " + ID + ")");
//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE (this.id = :idParam)");
            Query query = em.createQuery("FROM NoAliasEntity WHERE ( id = :idParam )");
            query.setParameter("idParam", ID);

//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE ( this.name = :nameParam )");
//            query.setParameter("nameParam", NAME);



//            Query query = em.createQuery("FROM NoAliasEntity WHERE ( name = :nameParam )"); //FIXED IN jakarta_data_fixes
//            query.setParameter("nameParam", NAME);


//            Query query = em.createQuery("FROM NoAliasEntity WHERE name = :nameParam");


//OK            Query query = em.createQuery("FROM NoAliasEntity WHERE name = :nameParam ");
//            query.setParameter("nameParam", NAME);

            NoAliasEntity result = (NoAliasEntity)query.getSingleResult();

            assertEquals(ID, result.getId());
            assertEquals(NAME, result.getName());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTestUpdateWhereWithSubquery() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();

        try {
            Query query = em.createQuery("UPDATE NoAliasEntity SET name = 'some new value' WHERE (SELECT COUNT(n) FROM NoAliasEntity n) > 0");
            int updated = query.executeUpdate();
            assertEquals(true, updated > 0);
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    //GitHub bug #2182
    @Test
    public void queryTestSelectAndGeneratedThis() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();

        try {
//            Query query = em.createQuery("SELECT this.name FROM NoAliasEntity WHERE name IS NOT NULL AND id = :idParam");
            Query query = em.createQuery("SELECT name FROM NoAliasEntity WHERE name IS NOT NULL AND id = :idParam");
            query.setParameter("idParam", ID);
            String result = (String)query.getSingleResult();
            assertEquals(NAME, result);
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    //GitHub bug #2188
    @Test
    public void queryTestSelectWhereWithEmbeddableOrRelationalAttributes() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();

        try {
            //FROM Business WHERE location.address.city=?1 ORDER BY name
//            Query query = em.createQuery("SELECT d FROM TestEntityDetail d WHERE d.master.id = :idParam ORDER BY d.name");
//            Query query = em.createQuery("FROM TestEntityDetail WHERE this.master.id = :idParam ORDER BY this.name");
            Query query = em.createQuery("FROM TestEntityDetail WHERE master.id = :idParam ORDER BY this.name");

            query.setParameter("idParam", ID);
            List<TestEntityDetail> result = (List<TestEntityDetail>)query.getResultList();
            assertEquals(2, result.size());
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    //GitHub bug #2189
    @Test
    public void queryTestSelectWhereWithIDFunction() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();

        try {
            //SELECT ID(THIS) FROM Prime o WHERE (o.name = :numberName OR :numeral=o.romanNumeral OR o.hex =:hex OR ID(THIS)=:num) ORDER BY o.numberId
//            Query query = em.createQuery("SELECT ID(this) FROM NoAliasEntity o WHERE o.name = 'some new value' OR ID(this) = :idParam");
//            Query query = em.createQuery("SELECT ID(o) FROM NoAliasEntity o WHERE o.name = 'some new value' OR ID(o) = :idParam ORDER BY o.id");
//            Query query = em.createQuery("SELECT ID(this) FROM NoAliasEntity this WHERE this.name = 'some new value' OR ID(this) = :idParam ORDER BY this.id");
            Query query = em.createQuery("SELECT ID(THIS) FROM NoAliasEntity WHERE this.name = 'some new value' OR ID(THIS) = :idParam ORDER BY id");



            query.setParameter("idParam", ID);
            Long result = (Long)query.getSingleResult();
            assertEquals(1L, result);
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    //GitHub bug #2184
    @Test
    public void queryTestUpdateWhereWithIDFunction() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();

        try {
            Query query = em.createQuery("UPDATE NoAliasEntity SET name = 'some new value' WHERE ID(this) = :idParam");
//            Query query = em.createQuery("UPDATE NoAliasEntity this SET this.name = 'some new value' WHERE ID(this) = :idParam");
            query.setParameter("idParam", ID);
            int updated = query.executeUpdate();
            assertEquals(true, updated > 0);
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    //GitHub bug #2192
    @Test
    public void queryTestSelectWithAggregateFunctionsAndGeneratedThis() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();

        try {
//            Query query = em.createQuery("SELECT MAX(this.id) FROM NoAliasEntity this");
            Query query = em.createQuery("SELECT MAX(id) FROM NoAliasEntity");
            long result = (long)query.getSingleResult();
            assertEquals(1L, result);
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    //GitHub bug #2192
    @Test
    public void testGenerateThisPrefixDetection() {
        String query = null;

        query = "SELECT name FROM NoAliasEntity WHERE name IS NOT NULL AND id = :idParam";
        assertTrue(generateImplicitThisAliasDetection(query));

        query = "SELECT name FROM NoAliasEntity ";
        assertTrue(generateImplicitThisAliasDetection(query));

        query = "SELECT e.name FROM NoAliasEntity e WHERE e.name IS NOT NULL AND e.id = :idParam";
        assertFalse(generateImplicitThisAliasDetection(query));

        query = "SELECT e.name FROM NoAliasEntity e";
        assertFalse(generateImplicitThisAliasDetection(query));

        query = "SELECT e.name FROM NoAliasEntity AS e";
        assertFalse(generateImplicitThisAliasDetection(query));

        query = "UPDATE Order SET length = length + 1";
        assertFalse(generateImplicitThisAliasDetection(query));

        query = "SELECT TRIM(TRAILING 'a' FROM e.firstName) FROM Employee e";
        assertFalse(generateImplicitThisAliasDetection(query));

    }

    @Test
    public void queryTestSelectWithAdditionalCriteria() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();
        em.setProperty("ID_PARAM", 1);
        try {
            Query query = em.createQuery("SELECT e FROM NoAliasEntityAdditionalCriteria");
            List<NoAliasEntityAdditionalCriteria> results = query.getResultList();
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    //GitHub bug #2297
    @Test
    public void queryTestSelectAndGeneratedThisOrderByIdFunction() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();
        em.getTransaction().begin();

        try {
            Query query = em.createQuery("SELECT ID(THIS) FROM NoAliasEntity WHERE ID(THIS) < ?1 ORDER BY ID(THIS) DESC");
            query.setParameter(1, ID + 10);
            List<Long> results = query.getResultList();
            assertEquals(2, results.size());
            em.getTransaction().rollback();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        } finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    //GitHub bug #2192 - helper
    //Check if this entity alias should be generated automatically
    private boolean generateImplicitThisAliasDetection(String jpqlFragment) {
        String jpqlFragmentLowerCase = jpqlFragment.toLowerCase(Locale.ROOT);
        String fromLowerCase = Expression.FROM.toLowerCase(Locale.ROOT);
        int count = 0;
        int fromIndex = jpqlFragmentLowerCase.indexOf(fromLowerCase);
        int formOccurrences = Collections.frequency(Arrays.asList(jpqlFragmentLowerCase.split(" ")), fromLowerCase);
        if (fromIndex >= 1 && formOccurrences == 1) {
            String jpqlFragmentFrom = jpqlFragment.substring(fromIndex);
            WordParser wordParser = new WordParser(jpqlFragmentFrom);
            wordParser.moveForwardIgnoreWhitespace(Expression.FROM);
            wordParser.skipLeadingWhitespace();
            while (!wordParser.word().equalsIgnoreCase(Expression.WHERE.toLowerCase(Locale.ROOT)) && !wordParser.word().isEmpty()) {
                count++;
                wordParser.moveForwardIgnoreWhitespace(wordParser.word());
                wordParser.skipLeadingWhitespace();
            }
        }
        return count == 1;
    }

    private CharSequence preParse(CharSequence jpqlFragment) {
        WordParser wordParser = new WordParser(jpqlFragment);
        wordParser.skipLeadingWhitespace();
        if (Expression.FROM.equalsIgnoreCase(wordParser.word())) {
            return Expression.SELECT + " " + Expression.THIS + " " + jpqlFragment;
        }
        return jpqlFragment;
    }
}
