package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.EntityTransaction;
import jakarta.persistence.Persistence;
import org.junit.jupiter.api.Test;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class TestRebateIdClassRecord {

    private static final String PU_NAME = "test-jpa-pu-rebate-derby";

    @Test
    public void init() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        EntityManager em = entityManagerFactory.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        List<Rebate> paidRebates = null;

        try {
            tx.begin();
            em.createQuery("DELETE FROM Rebate").executeUpdate();
            tx.commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
        }

        try {
            Rebate r1 = Rebate.of(10.00, "testOLGH28920", LocalTime.now().minusHours(1), LocalDate.now(), Rebate.Status.SUBMITTED,
                    LocalDateTime.now(), 1);
            Rebate r2 = Rebate.of(12.00, "testOLGH28920", LocalTime.now().minusHours(1), LocalDate.now(), Rebate.Status.PAID,
                    LocalDateTime.now(), 2);
            Rebate r3 = Rebate.of(14.00, "testOLGH28920", LocalTime.now().minusHours(1), LocalDate.now(), Rebate.Status.PAID,
                    LocalDateTime.now(), 2);

            tx.begin();
            em.persist(r1);
            em.persist(r2);
            em.persist(r3);
            tx.commit();

        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }


    @Test
    // @Ignore("Reference issue: https://github.com/OpenLiberty/open-liberty/issues/28920")
    public void testOLGH28920() {

        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        EntityManager em = entityManagerFactory.createEntityManager();
        EntityTransaction tx = em.getTransaction();
        List<Rebate> paidRebates = null;

        tx.begin();
        try {

            paidRebates = em.createQuery(
                            "SELECT NEW com.oracle.jpa.bugtest.Rebate(id, amount, customerId, purchaseMadeAt, purchaseMadeOn, status, updatedAt, version) "
                                    + "FROM Rebate "
                                    + "WHERE customerId=?1 AND status=com.oracle.jpa.bugtest.Rebate.Status.PAID "
                                    + "ORDER BY amount DESC, id ASC",
                            Rebate.class)
                    .setParameter(1, "testOLGH28920")
                    .getResultList();

/*
            paidRebates = em.createQuery(
                            "SELECT NEW com.oracle.jpa.bugtest.Rebate(this.id, this.amount, this.customerId, this.purchaseMadeAt, this.purchaseMadeOn, this.status, this.updatedAt, this.version) "
                                    + "FROM Rebate this "
                                    + "WHERE this.customerId=?1 AND this.status=com.oracle.jpa.bugtest.Rebate.Status.PAID "
                                    + "ORDER BY this.amount DESC, this.id ASC",
                            Rebate.class)
                    .setParameter(1, "testOLGH28920")
                    .getResultList();
*/
/*
            paidRebates = em.createQuery(
                            "SELECT OBJECT(this) "
                                    + "FROM Rebate this "
                                    + "WHERE this.customerId=?1 AND this.status=com.oracle.jpa.bugtest.Rebate.Status.PAID",
                            Rebate.class)
                    .setParameter(1, "testOLGH28920")
                    .getResultList();
*/

/*
            paidRebates = em.createQuery(
                            "SELECT OBJECT(this) "
                                    + "FROM Rebate this "
                                    + "WHERE this.status=com.oracle.jpa.bugtest.Rebate.Status.PAID",
                            Rebate.class)
                    .getResultList();
*/

            assertEquals(2, paidRebates.size());

            tx.commit();
        } catch (Exception e) {
            tx.rollback();
            throw e;
        }

/*
        assertEquals(2, paidRebates.size());
        assertEquals(14.00, paidRebates.get(0).amount, 0.001);
*/
    }

}
