package com.oracle.jpa.bugtest;

import jakarta.persistence.EntityManager;
import jakarta.persistence.EntityManagerFactory;
import jakarta.persistence.Persistence;
import org.eclipse.persistence.exceptions.ConversionException;
import org.eclipse.persistence.internal.helper.ClassConstants;
import org.eclipse.persistence.internal.helper.ConversionManager;
import org.junit.jupiter.api.Test;

import java.sql.Timestamp;
import java.time.Instant;
import java.time.Year;
import java.util.Date;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.fail;

public class TestTimestamp {

    private static final String PU_NAME = "test-jpa-pu-derby";

    private final long ID = 1L;
    private final Instant INSTANT_VALUE = Instant.ofEpochSecond(123456789L, 654321L);
    private final Year YEAR_VALUE = Year.of(2024);
    private final Date DATE_VALUE = Date.from(INSTANT_VALUE);
    private final Timestamp TIMESTAMP_VALUE = java.sql.Timestamp.valueOf("2006-02-11 15:20:30");

    @Test
    public void persistTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            em.getTransaction().begin();
            boolean value = false;
            TimestampEntity entity = new TimestampEntity();
            entity.setId(ID);
            entity.setInstantData(INSTANT_VALUE);
            entity.setYearData(YEAR_VALUE);
            entity.setDateData(DATE_VALUE);
            entity.setTsData(TIMESTAMP_VALUE);
            em.persist(entity);
            em.getTransaction().commit();
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void findTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            TimestampEntity entity = em.find(TimestampEntity.class, ID);
            assertNotNull(entity);

            assertEquals(INSTANT_VALUE, entity.getInstantData());
            assertEquals(YEAR_VALUE, entity.getYearData());
            // Time is not stored in DB
            // assertEquals(DATE_VALUE, entity.getDateData());
            assertEquals(TIMESTAMP_VALUE, entity.getTsData());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }

    @Test
    public void queryTest() {
        EntityManagerFactory entityManagerFactory = Persistence.createEntityManagerFactory(PU_NAME);
        assertNotNull(entityManagerFactory);

        EntityManager em = entityManagerFactory.createEntityManager();

        try {
            List<TimestampEntity> result = em.createQuery("SELECT OBJECT(o) FROM TimestampEntity o WHERE o.instantData = :instantData").
            setParameter("instantData", INSTANT_VALUE).
            getResultList();

            for (TimestampEntity entity: result) {
                assertNotNull(entity);

                assertEquals(INSTANT_VALUE, entity.getInstantData());
                assertEquals(YEAR_VALUE, entity.getYearData());
                // Time is not stored in DB
                // assertEquals(DATE_VALUE, entity.getDateData());
                assertEquals(TIMESTAMP_VALUE, entity.getTsData());
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException();
        }
        finally {
            if (em.getTransaction().isActive()) {
                em.getTransaction().rollback();
            }
            em.close();
        }
    }
}
